//
//  MessageForwardView.swift
//  SampleApp
//
//  Created by Nishan on 11/10/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct MessageForwardView: View {
    
    @StateObject var viewModel: MessageForwardViewModel
    
    @State private var subchannelId: String = ""
    @State private var statusLabel: String = ""
    
    var body: some View {
        VStack(alignment: .leading) {
            Text("SubChannel")
                .font(.footnote)
                .padding(.top, 8)
                
            TextField("Enter SubChannel Id", text: $subchannelId)
                .padding()
                .background(Color(uiColor: .secondarySystemBackground))
                .cornerRadius(12)
            
            Text("Message")
                .font(.footnote)
                .padding(.top, 8)
            Text(viewModel.messageText)
                .font(.subheadline)
                .fontWeight(.regular)
                .foregroundColor(Color.white)
                .padding()
                .background(Color(uiColor: .blue))
                .cornerRadius(12)
            
            Text("File Id (If any)")
                .font(.footnote)
                .padding(.top, 8)
            Text(viewModel.fileId)
                .padding()
                .background(Color(uiColor: .secondarySystemBackground))
                .cornerRadius(12)
            
            Button {
                Task {
                    do {
                        try await viewModel.forwardMessage(subchannelId: subchannelId)
                        statusLabel = "Message forwarded successfully!"
                        subchannelId = ""
                    } catch let error {
                        statusLabel = error.localizedDescription
                    }
                }
                
            } label: {
                
                Label("Forward", systemImage: "paperplane.fill")
                    .padding(16)
                    .background(Color(uiColor: .secondarySystemBackground))
                    .cornerRadius(12)
                    .font(.subheadline)
                    .foregroundColor(.black)

            }
            .frame(maxWidth: .infinity, alignment: .center)
            .padding(.top, 32)
            
            Text("Note:\nThis creates a new message with same text in provided subchannel!. If any media is attached, it will copy it too.")
                .multilineTextAlignment(.leading)
                .font(.footnote)
                .padding(.top, 8)
                .foregroundColor(.secondary)
            
            Text(statusLabel)
                .multilineTextAlignment(.leading)
                .font(.footnote)
                .padding(.top, 8)
                .foregroundColor(.blue)
        }
        .padding(.horizontal)
    }
}

class MessageForwardViewModel: ObservableObject {
    
    let message: AmityMessage
    let messageText: String
    let fileId: String
    
    init(message: AmityMessage) {
        self.message = message
        self.messageText = message.data?["text"] as? String ?? ""
                
        switch message.messageType {
        case .image:
            self.fileId = message.getImageInfo()?.fileId ?? ""
        case .file:
            self.fileId = message.getFileInfo()?.fileId ?? ""
        case .audio:
            self.fileId = message.getFileInfo()?.fileId ?? ""
        case .video:
            self.fileId = message.getVideoInfo()?.fileId ?? ""
        case .text, .custom:
            self.fileId = ""
        @unknown default:
            self.fileId = ""
        }
    }
    
    @MainActor
    func forwardMessage(subchannelId: String) async throws {
        guard !subchannelId.isEmpty else { return }
        
        let repository = AmityMessageRepository(client: AmityManager.shared.client!)
        
        let messageType = message.messageType
        
        switch messageType {
        case .image:
            let imageData = message.getImageInfo()
            guard let attachedFileId = imageData?.fileId else { fatalError() }
            
            let options = AmityImageMessageCreateOptions(subChannelId: subchannelId, attachment: .fileId(id: attachedFileId))
            try await repository.createImageMessage(options: options)
            
        case .file:
            let fileData = message.getFileInfo()
            guard let attachedFileId = fileData?.fileId else { fatalError() }
            
            let options = AmityFileMessageCreateOptions(subChannelId: subchannelId, attachment: .fileId(id: attachedFileId))
            try await repository.createFileMessage(options: options)

        case .audio:
            let fileData = message.getFileInfo()
            guard let attachedFileId = fileData?.fileId else { fatalError() }
            
            let options = AmityAudioMessageCreateOptions(subChannelId: subchannelId, attachment: .fileId(id: attachedFileId))
            try await repository.createAudioMessage(options: options)

        case .video:
            let fileData = message.getVideoInfo()
            guard let attachedFileId = fileData?.fileId else { fatalError() }
            
            let options = AmityVideoMessageCreateOptions(subChannelId: subchannelId, attachment: .fileId(id: attachedFileId))
            try await repository.createVideoMessage(options: options)
        case .text:
            let options = AmityTextMessageCreateOptions(subChannelId: subchannelId, text: messageText)
            try await repository.createTextMessage(options: options)

        case .custom:
            let options = AmityCustomMessageCreateOptions(subChannelId: subchannelId, data: message.data ?? [:])
            try await repository.createCustomMessage(options: options)
            
        @unknown default:
            break
        }
    }
}

// 0BC314
