//
//  BlockedUserListViewController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 10/4/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

class BlockedUserListViewController: UIViewController {

    let tableView = UITableView()

    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    let feedbackImpactGenerator = UIImpactFeedbackGenerator()
    
    var listManager: UserListManager!
    var userRepository: AmityUserRepository!
    var userRelationship: AmityUserRelationship!
    var userListToken: AmityNotificationToken?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(tableView)
        NSLayoutConstraint.activate([
            view.leadingAnchor.constraint(equalTo: tableView.leadingAnchor, constant: 0),
            view.trailingAnchor.constraint(equalTo: tableView.trailingAnchor, constant: 0),
            view.topAnchor.constraint(equalTo: tableView.topAnchor, constant: 0),
            view.bottomAnchor.constraint(equalTo: tableView.bottomAnchor, constant: 0)
        ])
        
        listManager = UserListManager(client: AmityManager.shared.client!)
        userRepository = AmityUserRepository(client: AmityManager.shared.client!)
        
        tableView.register(UINib(nibName: "BlockedUserListCell", bundle: nil), forCellReuseIdentifier: BlockedUserListCell.identifier)
        
        setupViews()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        fetchBlockedUserList()
    }
    
    func updateUserList() {
        self.tableView.reloadData()
    }
    
    func setupViews() {
        self.title = "Blocked Users"
        
        tableView.dataSource = self
        tableView.delegate = self
        tableView.reloadData()
        tableView.keyboardDismissMode = .onDrag
    }
    
    func fetchBlockedUserList() {
        listManager.fetchBlockedUserList { [weak self] in
            self?.updateUserList()
        }
    }
    
    func scrollViewWillEndDragging(_ scrollView: UIScrollView, withVelocity velocity: CGPoint, targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        
        let frameHeight = scrollView.frame.size.height
        let contentOffset = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height
        
        let distanceFromBottom = contentHeight - contentOffset
        if distanceFromBottom < frameHeight {
            listManager.loadMoreUsers()
        }
    }
    
    func displayUserFeed(userId: String, userName: String?) {
        let feedManager = UserPostsFeedManager(client: AmityManager.shared.client!, userId: userId, userName: userName)
        feedManager.feedType = .userFeed
        
        let postsFeedStoryboard = UIStoryboard(name: "Feed", bundle: nil)
        let postsFeedController = postsFeedStoryboard.instantiateViewController(withIdentifier: UserPostsFeedViewController.identifier) as! UserPostsFeedViewController
        postsFeedController.feedManager = feedManager
        
        self.navigationController?.pushViewController(postsFeedController, animated: true)
    }
}

extension BlockedUserListViewController: UITableViewDataSource, UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return listManager.numberOfUsers()
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: BlockedUserListCell.identifier) as! BlockedUserListCell
        
        let user = listManager.getUserItem(at: indexPath.row)
        cell.userNameLabel.text = user?.displayName ?? "-"
        cell.userIdLabel.text = "Id: \(user?.userId ?? "")"
        cell.avatarLabel.text = "Avatar: Id: \(user?.avatarFileId ?? ""), URL: \(user?.avatarCustomUrl ?? "")"
        cell.infoLabel.text = "IsDeleted: \(user?.isDeleted ?? false)"
        cell.cellOptionTapAction = { [weak self] in
            self?.showAvailableActions(for: cell)
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        /* Uncomment if needed!
        
        guard let user = listManager.getUserItem(at: indexPath.row) else { return }
        
        var actions = [UIAlertAction]()
        actions.append(UIAlertAction(title: "View User Feed", style: .default, handler: { [weak self] action in
            self?.displayUserFeed(userId: user.userId, userName: user.displayName)
        }))
        
        actions.append(UIAlertAction(title: "Real Time Event", style: .default, handler: { [weak self] action in
            self?.displayRealTimeEventScreen(user: user)
        }))
        
        actions.append(UIAlertAction(title: "Flag User", style: .default, handler: { [weak self] action in
            self?.flagUser(user: user)
        }))
        
        actions.append(UIAlertAction(title: "Unflag User", style: .default, handler: { [weak self] action in
            self?.unflagUser(user: user)
        }))
        
        AppUtility.showActionSheet(in: self, title: "", message: "What do you want to do?", actions: actions)
         
         */
    }
    
    func unblockUser(userId: String) {
        Task { @MainActor in
            let message: String
            do {
                try await userRelationship.unblockUser(userId: userId)
                message = "User \(userId) is unblocked"
            } catch let error {
                message = "Error: \(error.localizedDescription)"
            }
            
            AppUtility.showAlert(in: self, title: "", message: message, action: nil)
        }
    }
}

extension BlockedUserListViewController {
    
    func showAvailableActions(for cell: BlockedUserListCell) {
        guard let indexPath = tableView.indexPath(for: cell),
              let userId = listManager.getUserItem(at: indexPath.row)?.userId else { return }
        
        feedbackImpactGenerator.impactOccurred()
        
        userRelationship = userRepository.userRelationship
        userListToken?.invalidate()
        
        userListToken = userRelationship.getFollowInfo(withUserId: userId).observeOnce { [weak self] liveObject, error in
            
            guard let info = liveObject.object, liveObject.dataStatus == .fresh, let weakSelf = self else { return }
            
            // Invalidate this token
            self?.userListToken?.invalidate()
            self?.userListToken = nil
            
            var actionsList = [UIAlertAction]()
            
            let followAction = UIAlertAction(title: "Follow", style: .default) { [weak self] _ in
                Task { @MainActor in
                    do {
                        let result = try await self?.userRelationship.follow(withUserId: userId)
                        self?.displayFollowActionAlert(action: .follow, status: result?.0 ?? false, error: nil)
                    } catch {
                        self?.displayFollowActionAlert(action: .follow, status: false, error: error)
                    }
                }
            }
            
            let unblockAction = UIAlertAction(title: "Unblock", style: .default) { [weak self] _ in
                self?.unblockUser(userId: userId)
            }
            
            actionsList = [followAction, unblockAction]

            AppUtility.showActionSheet(in: weakSelf, title: "Actions", message: "", actions: actionsList)
        }
    }
    
    func displayFollowActionAlert(action: FollowAction, status: Bool, error: Error?) {
        let title: String = status == true ? "\(action.rawValue.capitalized) Successful" : "\(action.rawValue.capitalized) Failed"
        let message: String = error == nil ? "" : error?.localizedDescription ?? ""
        
        // Show this alert after 1 second.
        AppUtility.showAlert(in: self, title: title, message: message, action: nil)
    }
}
